/* Copyright (c) 2012 imacat
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * SpreadsheetDocument
 * 
 * Created on 2012-08-16, rewritten from CalcMosaic
 * 
 * Copyright (c) 2012 imacat
 */

package tw.idv.imacat.calcmosaic;

import java.awt.Dimension;
import java.util.ResourceBundle;

import com.sun.star.awt.Size;
import com.sun.star.awt.XView;
import com.sun.star.awt.XWindow;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XIndexAccess;
import com.sun.star.container.XNameContainer;
import com.sun.star.frame.XController;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XModel;
import com.sun.star.lang.XComponent;
import com.sun.star.script.XLibraryContainer;
import com.sun.star.script.provider.XScript;
import com.sun.star.script.provider.XScriptProvider;
import com.sun.star.script.provider.XScriptProviderSupplier;
import com.sun.star.sheet.XSpreadsheetDocument;
import com.sun.star.table.XColumnRowRange;
import com.sun.star.table.XTableRows;
import com.sun.star.uno.UnoRuntime;

/**
 * A spreadsheet document.
 * 
 * @author <a href="mailto:imacat&#64;mail.imacat.idv.tw">imacat</a>
 * @version 1.0.2
 */
public class SpreadsheetDocument {
    
    /** The BASIC library name. */
    private static final String libraryName = "Standard";
    
    /** The localization resources. */
    private ResourceBundle l10n = null;
    
    /** The spreadsheet document. */
    private XSpreadsheetDocument document = null;
    
    /** The document controller. */
    private XController xController = null;
    
    /**
     * Creates a new instance of SpreadsheetDocument.
     * 
     * @param document the document component
     */
    public SpreadsheetDocument(XComponent document) {
        XModel xModel = null;
        XIndexAccess sheets = null;
        
        // Obtains the localization resources
        this.l10n = ResourceBundle.getBundle(
            this.getClass().getPackage().getName() + ".res.L10n");
        
        this.document = (XSpreadsheetDocument)
            UnoRuntime.queryInterface(
            XSpreadsheetDocument.class, document);
        
        // Obtains the document controller
        xModel = (XModel) UnoRuntime.queryInterface(
            XModel.class, this.document);
        this.xController = xModel.getCurrentController();
    }
    
    /**
     * Adds a module of BASIC macros to the document.
     * 
     * @param module the name of the module.
     * @param macros the content of the module
     */
    public void addBasicModule(String module, String macros) {
        XPropertySet xProps = null;
        Object libraries = null;
        XLibraryContainer libraryContainer = null;
        Object library = null;
        XNameContainer moduleContainer = null;
        
        xProps = (XPropertySet) UnoRuntime.queryInterface(
            XPropertySet.class, document);
        try {
            libraries = xProps.getPropertyValue("BasicLibraries");
        } catch (com.sun.star.beans.UnknownPropertyException e) {
            throw new java.lang.RuntimeException(e);
        } catch (com.sun.star.lang.WrappedTargetException e) {
            throw new java.lang.RuntimeException(e);
        }
        libraryContainer = (XLibraryContainer)
            UnoRuntime.queryInterface(
            XLibraryContainer.class, libraries);
        if (!libraryContainer.hasByName(libraryName)) {
            try {
                libraryContainer.createLibrary(libraryName);
            } catch (com.sun.star.lang.IllegalArgumentException e) {
                throw new java.lang.IllegalArgumentException(e);
            } catch (com.sun.star.container.ElementExistException e) {
                throw new java.lang.RuntimeException(e);
            }
        }
        try {
            library = libraryContainer.getByName(libraryName);
        } catch (com.sun.star.container.NoSuchElementException e) {
            throw new java.lang.RuntimeException(e);
        } catch (com.sun.star.lang.WrappedTargetException e) {
            throw new java.lang.RuntimeException(e);
        }
        moduleContainer = (XNameContainer) UnoRuntime.queryInterface(
            XNameContainer.class, library);
        if (moduleContainer.hasByName(module)) {
            try {
                moduleContainer.removeByName(module);
            } catch (com.sun.star.container.NoSuchElementException e) {
                throw new java.lang.RuntimeException(e);
            } catch (com.sun.star.lang.WrappedTargetException e) {
                throw new java.lang.RuntimeException(e);
            }
        }
        try {
            moduleContainer.insertByName(module, macros);
        } catch (com.sun.star.lang.IllegalArgumentException e) {
            throw new java.lang.IllegalArgumentException(e);
        } catch (com.sun.star.container.ElementExistException e) {
            throw new java.lang.RuntimeException(e);
        } catch (com.sun.star.lang.WrappedTargetException e) {
            throw new java.lang.RuntimeException(e);
        }
    }
    
    /**
     * Runs a BASIC macro of the document.
     * 
     * @param module the name of the module of the macro.
     * @param macro the name of the macro.
     * @param param the parameters for the macro
     */
    public void runBasicMacro(String module, String macro, Object param[]) {
        String scriptUri = String.format(
            "vnd.sun.star.script:%s.%s.%s?language=Basic&location=document",
            libraryName, module, macro);
        XScriptProviderSupplier xSupplier = (XScriptProviderSupplier)
            UnoRuntime.queryInterface(
            XScriptProviderSupplier.class, document);
        XScriptProvider xProvider = xSupplier.getScriptProvider();
        XScript xScript = null;
        short outParamIndex[][] = new short[0][0];
        Object outParam[][] = new Object[0][0];
        
        try {
            xScript = xProvider.getScript(scriptUri);
        } catch (com.sun.star.script.provider.ScriptFrameworkErrorException e) {
            throw new java.lang.IllegalArgumentException(e);
        }
        try {
            xScript.invoke(param, outParamIndex, outParam);
        } catch (com.sun.star.script.provider.ScriptFrameworkErrorException e) {
            throw new java.lang.RuntimeException(e);
        } catch (com.sun.star.reflection.InvocationTargetException e) {
            throw new java.lang.RuntimeException(e);
        } catch (com.sun.star.lang.DisposedException e) {
        }
    }
    
    /**
     * Returns the size of the window.
     * 
     * @return the size of the window
     */
    public Dimension getWindowSize() {
        XFrame frame = this.xController.getFrame();
        XWindow window = frame.getComponentWindow();
        XView xView = (XView) UnoRuntime.queryInterface(
            XView.class, window);
        Size size = xView.getSize();
        return new Dimension(size.Width - 55, size.Height - 72);
    }
    
    /**
     * Returns the height of the rows.  We do not need
     * the width, since the height is always shorter in new
     * spreadsheets.
     * 
     * @return the height of the rows.
     */
    public int getRowHeight() {
        XIndexAccess sheets = (XIndexAccess)
            UnoRuntime.queryInterface(
            XIndexAccess.class, this.document.getSheets());
        Object sheet = null;
        XColumnRowRange xColumnRowRange = null;
        XTableRows rows = null;
        XPropertySet xProps = null;
        int height = -1;
        
        try {
            sheet = sheets.getByIndex(0);
        } catch (com.sun.star.lang.IndexOutOfBoundsException e) {
            throw new java.lang.IndexOutOfBoundsException(
                e.getMessage());
        } catch (com.sun.star.lang.WrappedTargetException e) {
            throw new java.lang.RuntimeException(e);
        }
        
        // Obtains the height of the rows.
        xColumnRowRange = (XColumnRowRange) UnoRuntime.queryInterface(
            XColumnRowRange.class, sheet);
        rows = xColumnRowRange.getRows();
        xProps = (XPropertySet) UnoRuntime.queryInterface(
            XPropertySet.class, rows);
        try {
            height = (Integer) xProps.getPropertyValue("Height");
        } catch (com.sun.star.beans.UnknownPropertyException e) {
            throw new java.lang.RuntimeException(e);
        } catch (com.sun.star.lang.WrappedTargetException e) {
            throw new java.lang.RuntimeException(e);
        }
        
        return height;
    }
    
    /**
     * Gets a string for the given key from this resource bundle
     * or one of its parents.  If the key is missing, returns
     * the key itself.
     * 
     * @param key the key for the desired string 
     * @return the string for the given key 
     */
    private String _(String key) {
        try {
            return this.l10n.getString(key);
        } catch (java.util.MissingResourceException e) {
            return key;
        }
    }
}
